//  Beispiel 7.3 "Erweiterung des Sketches 7.2 um automatischen WLAN-Aufbau"
//               "Darüber hinaus werden Seiten-Elemente mit Funktionen generiert"
//               "und die zuletzt angeklickten Einstellungen bleiben angeklickt."
//               "Weiterhin sind zwei neue Action-Buttons implementiert worden"
//               "um den IoT-Brick neu zu starten und auszuschalten."
//
//  Unter Verwendung der IoT Brick Library

#include <all_IoT.h>


int counter = 0;
int currentGPIO = 0;                                                // 0 = Beide aus, 1 = GPIO13 ein, 2 = GPIO14 ein, 3 = Beide ein
int unicodeMode = 1;                                                // 0 = Asc II Namen, 1 = Unicode Namen
int displayMode = 1;                                                // 0 = Display aus,  1 = Display ein
int colorMode = 0;                                                  // 0 = Rote Schrift, 1 = Grüne Schrift, 2 = Blaue Schrift usw.

String vorname_web    = "(Vorname)";
String nachname_web   = "(Nachname)";


void setup()
{
  IoT_Init(true);
  IoT_TerminalWaitInit(true);
  IoT_WLANautoConnect(true);                                        // Verbindung, ggf. gespeicherte, über WLAN herstellen
  IoT_NTPinit();

  IoT_WebServer.on("/", handleRoot);                                // Browser-Handler (siehe unten) für das Stammverzeichnis
  IoT_WebServer.on("/form01", handleForm01);                        // Handler für die Eingabe der Namen in den Eingabefeldern
  IoT_WebServer.on("/form02", handleForm02);                        // Handler für "IoT-Brick neu starten"
  IoT_WebServer.on("/form03", handleForm03);                        // Handler für "IoT-Brick ausschalten"
  IoT_WebServer.begin();                                            // ab jetzt "horcht" der Server auf HTTP-Anfragen
  Serial.println("HTTP server started");
}

void loop()
{
  IoT_Idle();

  IoT_WebServer.handleClient();                                     // Bediene die http Anfragen
  
  if (IoT_Keypress())                                               // Wenn Taster gedrückt wird, Konfiguration zeigen
  {
    IoT_DisplayWLANstatus();
    if (counter % 10 == 0)                                          // Infos seriell nicht zu oft ausgeben (ca. 1 Sekunde)
    {
      Serial.println(IoT_NTPdatetime() + " (" + IoT_NTPdaylightSavingTime(true) + ")");
      Serial.print("WiFi is ");
      Serial.print(IoT_WLANconnected() ? "connected" : "not connected");
      Serial.println(". Uptime: " + IoT_WLANuptime(true) + " seit " + IoT_WLANstartDatetime());
    }
  }
  else
  {
    IoT_Idle();
    if (counter % 25 == 0) //Infos seriell nicht zu oft ausgeben (ca. alle 2 Sekunden)
    {
      if (IoT_NTPeventAvail)                                        // Zeitevent triggern
      {
        IoT_NTPprintEvent(IoT_NTPcurrentEvent);
        IoT_NTPeventAvail = false;
      }
    }
    
    IoT_DisplayClear(16);                                           // OLED Display löschen (16 Punkt Schrift)
    IoT_DisplayAlignText(TEXT_ALIGN_CENTER);                        // Zentrierte Darstellung des Textes
    if (IoT_NTPvalid())                                             // Prüfen, ob eine gültige Uhrzeit aus dem Internet empfangen wurde
    {
      if (displayMode == 1)
      {
        IoT_DisplayDrawText(63, 0, IoT_NTPtime());                  // Uhrzeit im Display anzeigen
        IoT_DisplayDrawText(63, 15, IoT_NTPdate());                 // Datum im Display anzeigen
        IoT_Idle();
      }
    }
    else                                                            // "Bitte warten..." zentriert anzeigen
    {
      if (displayMode == 1)
      {
        if (IoT_WLANinited)
        {
          IoT_DisplayClear(24);                                       // OLED Display löschen (24 Punkt Schrift)
          IoT_DisplayAlignText(TEXT_ALIGN_CENTER);                    // Zentrierte Darstellung des Textes
          IoT_DisplayDrawText(64, 5, "Bitte");
          IoT_DisplayDrawText(64, 34, "warten...");
        }
        else
        {
          IoT_DisplayClear(24);                                       // OLED Display löschen (24 Punkt Schrift)
          IoT_DisplayAlignText(TEXT_ALIGN_CENTER);                    // Zentrierte Darstellung des Textes
          IoT_DisplayDrawText(64, 5, "Kein");
          IoT_DisplayDrawText(64, 34, "WLAN");
        }
      }
    }
  }
  IoT_DisplayUpdate();                                              // OLED-Anzeige aktualisieren
  delay(100);                                                       // 100 ms. warten
  t_TerminalRead();                                                 // Reset-Anforderung prüfen
  counter++;
}

//Mit der Funktion handleRoot() wird die Website ausgeliefert sobald eine Anfrage von einem Browser eintrifft

void handleRoot ()
{
  String time_web = IoT_NTPtime();
  String date_web = IoT_NTPdate();
  String ipaddr_web  = IoT_WLANaddress(false);
  String title_web = "Brick'R'knowledge IoT Brick-Website";
  int sec = millis() / 1000;
  int min = sec / 60;
  int hr = min / 60;
  String ColorHexVal = "";
  switch (colorMode)
  {
    case 0:
      ColorHexVal = "#FF0000";
      break;
    case 1:
      ColorHexVal = "#00FF00";
      break;
    case 2:
      ColorHexVal = "#0000FF";
      break;
    case 3:
      ColorHexVal = "#00FFFF";
      break;
    case 4:
      ColorHexVal = "#FF00FF";
      break;
    case 5:
      ColorHexVal = "#FFFF00";
      break;
    case 6:
      ColorHexVal = "#000000";
      break;
    default:
      ColorHexVal = "#000000";
      break;
  }
  String content =                                                  // Die HTML-Seite in lesbarer Formatierung,
                                                                    // darf auch Variablen enthalten
"<html>\
  <head>\
    <title>" + title_web + "</title>\
    <style>\
      body { background-color: #FFFFFF; font-family: Menlo, Monaco, Courier, monospace; Color: " + ColorHexVal + "; }\
    </style>\
    <style type='text/css'>\
      h1 { font-family: Arial, 'Helvetica Neue', Helvetica, sans-serif; Color: #000088; }\
    </style>\
  </head>\
  <body>\
    <h1>" + title_web + "</h1>\
    <p>Datum: &nbsp; &nbsp;  &nbsp;  &nbsp;" + date_web + "</p>\
    <p>Uhrzeit: &nbsp;  &nbsp;  &nbsp;" + time_web + "</p>\
    <p>IP-Addresse: &nbsp;" + ipaddr_web + "</p>\
    <p>Letzter Name: " + vorname_web + " " + nachname_web + "</p>\
    " + IoT_WebFormOpen("form01") + "\
      " + IoT_WebInput("Vorname:  ", "vname", "", 20) + "<br>\
      " + IoT_WebInput("Nachname: ", "nname", "", 20) + "<br>\
      <br>\
      " + IoT_WebCheckBox(" Unicode-Zeichen im Namen erlauben", "unicode", "chkd", unicodeMode) + "<br>\
      " + IoT_WebCheckBox(" OLED-Display eingeschaltet", "display", "chkd", displayMode) + "<br>\
      <br>\
      " + IoT_WebRadioButton(" Keinen Ausgang einschalten", 0, "LEDgroup", "none",   currentGPIO) + "<br>\
      " + IoT_WebRadioButton(" GPIO 13 einschalten",        1, "LEDgroup", "GPIO13", currentGPIO) + "<br>\
      " + IoT_WebRadioButton(" GPIO 14 einschalten",        2, "LEDgroup", "GPIO14", currentGPIO) + "<br>\
      " + IoT_WebRadioButton(" Beide Ausgänge einschalten", 3, "LEDgroup", "both",   currentGPIO) + "<br>\
      <br>\
      " + IoT_WebRadioButton(" Rote Schrift",     0, "COLORgroup", "red",     colorMode) + "<br>\
      " + IoT_WebRadioButton(" Grüne Schrift",    1, "COLORgroup", "green",   colorMode) + "<br>\
      " + IoT_WebRadioButton(" Blaue Schrift",    2, "COLORgroup", "blue",    colorMode) + "<br>\
      " + IoT_WebRadioButton(" Türkise Schrift",  3, "COLORgroup", "cyan",    colorMode) + "<br>\
      " + IoT_WebRadioButton(" Violette Schrift", 4, "COLORgroup", "magenta", colorMode) + "<br>\
      " + IoT_WebRadioButton(" Gelbe Schrift",    5, "COLORgroup", "yellow",  colorMode) + "<br>\
      " + IoT_WebRadioButton(" Schwarze Schrift", 6, "COLORgroup", "black",   colorMode) + "<br>\
      <br>\
      " + IoT_WebFormSubmitButton("Namen im Terminal und auf der Webseite anzeigen, Farbe, GPIO & Display schalten") + "<br>\
    " + IoT_WebFormClose() + "\
    " + IoT_WebFormActionButton ("form02", "IoT-Brick neu starten") + "\
    " + IoT_WebFormActionButton ("form03", "IoT-Brick ausschalten") + "\
  </body>\
</html>";

  IoT_WebServer.send(200, "text/html", content);                    // HTTP-Statuscode 200 = OK (Anfrage wurde erfolgreich bearbeitet)
}

void handleForm01 ()
{
  Serial.println("Button wurde betätigt: Form01");
  IoT_WebPrintAllFields();                                          // Alle gefundenen Felder auf dem Terminal ausgeben
  if (IoT_WebTestField("vname"))                                    // Testen, ob ein Feld mit dem Namen "vname" existiert
  {
    vorname_web = IoT_WebGetField("vname");                         // Das Web-Feld "vname" auslesen
  }
  if (IoT_WebTestField("nname"))                                    // Testen, ob ein Feld mit dem Namen "vname" existiert
  {
    nachname_web = IoT_WebGetField("nname");                        // Das Web-Feld "nname" auslesen
  }
  unicodeMode = boolval(IoT_WebTestField("unicode"));               // Testen, ob die Checkbox mit dem Namen "unicode" angekreuzt wurde
  displayMode = boolval(IoT_WebTestField("display"));               // Testen, ob die Checkbox mit dem Namen "display" angekreuzt wurde
  if (IoT_WebTestField("LEDgroup"))                                 // Testen, ob Felder mit dem Namen "LEDgroup" existieren
  {
    String selection = IoT_WebGetField("LEDgroup");
    if (selection == "none")                                        // Beide Leitungen aus
    {
      digitalWrite(13, LOW);
      digitalWrite(14, LOW);
      currentGPIO = 0;
    }
    else if (selection == "GPIO13")                                 // GPIO 13
    {
      digitalWrite(13, HIGH);
      digitalWrite(14, LOW);
      currentGPIO = 1;
    }
    else if (selection == "GPIO14")                                 // GPIO 14
    {
      digitalWrite(13, LOW);
      digitalWrite(14, HIGH);
      currentGPIO = 2;
    }
    else if (selection == "both")                                   // Beide Leitungen ein
    {
      digitalWrite(13, HIGH);
      digitalWrite(14, HIGH);
      currentGPIO = 3;
    }
  }
  if (IoT_WebTestField("COLORgroup"))                               // Testen, ob Felder mit dem Namen "COLORgroup" existieren
  {
    String selection = IoT_WebGetField("COLORgroup");
    if (selection == "red")                                         // Rot
    {
      colorMode = 0;
    }
    else if (selection == "green")                                 // Grün
    {
      colorMode = 1;
    }
    else if (selection == "blue")                                   // Blau
    {
      colorMode = 2;
    }
    else if (selection == "cyan")                                   // Türkis
    {
      colorMode = 3;
    }
    else if (selection == "magenta")                                // Violett
    {
      colorMode = 4;
    }
    else if (selection == "yellow")                                 // Gelb
    {
      colorMode = 5;
    }
    else if (selection == "black")                                  // Schwarz
    {
      colorMode = 6;
    }
  }
  if (unicodeMode == 0)
  {
    vorname_web  = cleanasc(vorname_web);
    nachname_web = cleanasc(nachname_web);
  }
  handleRoot();                                                     // Wieder auf die Hauptseite zurück schalten
}

void handleForm02 ()
{
  Serial.println("Button wurde betätigt: Form02");
  handleRoot();                                                     // Wieder auf die Hauptseite zurück schalten
  t_Restart();
}

void handleForm03 ()
{
  Serial.println("Button wurde betätigt: Form03");
  handleRoot();                                                     // Wieder auf die Hauptseite zurück schalten
  IoT_ShutDown();
}

